// 网页环境
export enum WebEnvEnum {
  default = 0,
  ios = 1,
  android = 2,
  zshftIos = 3,
}


// 比较大小
abstract class CompareInit {
  protected prop: string|Array<string>;

  constructor(prop: string|Array<string>) {
    this.prop = prop;
  }

  protected init(): any {
    let prop = this.prop;
    let that: any = this;
    return function (obj_1: any, obj_2: any): any {
      let val_1: any|undefined, val_2: any|undefined;
      if (prop instanceof Array) {
        for (let info of prop) {
          val_1 = (val_1 || obj_1)[info];
          val_2 = (val_2 || obj_2)[info];
        }
      }
      val_1 = (val_1 instanceof Array ? val_1.length : obj_1[prop as string]);
      val_2 = (val_2 instanceof Array ? val_2.length : obj_2[prop as string]);
      if (!isNaN(Number(val_1)) && !isNaN(Number(val_2))) {
        val_1 = Number(val_1);
        val_1 = Number(val_1);
      }
      return that.initSuccess(val_1, val_2);
    }
  }

  abstract initSuccess(val_1: number, val_2: number): any
}
export class Compare extends CompareInit {
  private compareBoolean: boolean = true;

  constructor(prop: string|Array<string>) {
    super(prop);
  }

  initSuccess(val_1: number, val_2: number) {
    const eventMap: any = {
      true: 'maxToMinEvent',
      false: 'minToMaxEvent'
    };

    let that: any = this;
    let boolean: any = that.compareBoolean;
    let event: any = eventMap[boolean];
    return that[event](val_1, val_2);
  }

  public maxToMin(): any {
    this.compareBoolean = true;
    return super.init();
  }

  public minToMax(): any {
    this.compareBoolean = false;
    return super.init();
  }

  private maxToMinEvent(val_1: number, val_2: number): any {
    if (val_1 > val_2) return -1;

    if (val_1 < val_2) return 1;

    return 0;
  }

  private minToMaxEvent(val_1: number, val_2: number): any {
    if (val_1 < val_2) return -1;

    if (val_1 > val_2) return 1;

    return 0;
  }
}


// 工具
export class Utils {
  // 判断手机号是否正确
  checkTelephone(tel: string): boolean {
    return !!(/^1[23456789]\d{9}$/.test(tel));
  }

  //必须为字母加数字
  checkPassWord(password: string): boolean {
    let str = password;
    let reg1 = new RegExp(/^[0-9A-Za-z]+$/);
    if (!reg1.test(str)) {
      return false;
    }
    let reg = new RegExp(/[A-Za-z].*[0-9]|[0-9].*[A-Za-z]/);
    return !!reg.test(str);
  }

  // 截取url上参数值
  getQueryString(name: string): string | number | null {
    let reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)");
    let r = window.location.search.substr(1).match(reg);
    if (r != null) return unescape(r[2]);
    return null;
  }
  
  // 只保留数字小数点和减号
  replaceNumber(str: string): string {
    return str.replace(/[^\d.]/g,"");
  }
  
  /**
   * 格式化数字单位
   * @param val
   * @param onlyNumber 是否仅显示数字不显示单位
   */
  formatNumber(val: number, onlyNumber: boolean = false): string {
    val = val || 0;
    if (val/10000 >= 1) {
      return Number((val/10000).toFixed(2)) + (onlyNumber ? "" : "万");
    }
    
    return val.toString();
  }
  // 获取数字单位
  formatNumberUnit(val: number): string {
    if (val/10000 >= 1) {
      return "万";
    }
    
    return "";
  }
  
  // 判断网页环境
  checkWebEnv(): WebEnvEnum {
    let u = navigator.userAgent;
    let isAndroid = u.indexOf('Android') > -1 || u.indexOf('Adr') > -1; //android终端
    let isiOS = !!u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/); //ios终端
    if (isAndroid) return WebEnvEnum.android;
  
    if (isiOS) {
      return WebEnvEnum.ios;
    }
    
    if (isiOS) return WebEnvEnum.ios;
    
    return WebEnvEnum.default;
  }
}
