// 格式化时间
export class FormDate extends Date {
  private now: any;
  constructor(now: Date|string|number = new Date()) {
    super();
    if (typeof now === "string") {
      now = now.replace(/-/g, "/");
    }
    if (now instanceof Date) {
      this.now = now;
      return;
    }

    if(/^[0-9]*$/.test((now as any))){
      now = Number(now);
    }
    this.now = new Date(new Date(now));
  }
  
  // 获取时间戳
  getTimeStr(): number {
    return this.now.getTime();
  }

  getYear() {
    return (this.now.getYear() + 1900).toString()
  }

  getMonth() {
    let month = this.now.getMonth() + 1;
    return (month < 10 ? "0" + month : month.toString());
  }

  getDay() {
    let date = this.now.getDate();
    return (date < 10 ? "0" + date : date.toString());
  }

  getHour() {
    let hour = this.now.getHours();
    return (hour < 10 ? "0" + hour : hour.toString());
  }

  getMinute() {
    let minute = this.now.getMinutes();
    return (minute < 10 ? "0" + minute : minute.toString());
  }

  getSecond() {
    let second = this.now.getSeconds();
    return (second < 10 ? "0" + second : second.toString());
  }

  getBigTime(times: string|number): string {
    let formatTime, Time = new Date();
    if (typeof times === "string") {
      formatTime = times.replace(/-/g, '/');
      Time = new Date(formatTime);
    }
    if (typeof times === "number") {
      Time = new Date(times);
    }
    let timestamp = Time.getTime();
    let mistiming = Math.round((Date.now() - timestamp) / 1000);
    let arr = ['年', '月', '周', '天', '小时', '分钟', '秒'];
    let arrVal = [31536000, 2592000, 604800, 86400, 3600, 60, 1];
    for (let i = 0; i <= arrVal.length; i++) {
      let inm = Math.floor(mistiming / arrVal[i]);
      if (inm != 0) {
        return inm + arr[i] + '前';
      }
    }

    return "";
  }

  getStoreTime(times: string|number) {
    let timestamp = new Date().getTime();
    if (typeof times === "string") {
      timestamp = new Date(times.replace(/-/g, '/')).getTime()
    }
    if (typeof times === "number") {
      timestamp = times;
    }
    let dayStartTime = new Date().setHours(0, 0, 0, 0);
    let dayEndTime = new Date().setHours(23, 59, 59, 999);
    let timestampEndTime = new Date(timestamp).setHours(23, 59, 59, 999);
    let oneDayTimestamp = 24 * 60 * 60 * 1000;
    // 今天
    if (timestampEndTime === dayEndTime) {
      let dataObj = new FormDate(new Date(timestamp));
      return dataObj.getHour() + ":" + dataObj.getMinute();
    }

    if (oneDayTimestamp + timestampEndTime === dayEndTime) {
      return "昨天";
    }

    if (oneDayTimestamp * 2 + timestampEndTime === dayEndTime) {
      return "前天";
    }

    let arr = ['年', '月', '天', '小时', '分钟', '秒'];
    let arrVal = [31536000, 2592000, 86400, 3600, 60, 1];
    for (let i = 0; i <= arrVal.length; i++) {
      let inm = Math.floor((dayEndTime - timestampEndTime) / 1000 / arrVal[i]);
      if (inm != 0) {
        return inm + arr[i] + '前';
      }
    }
  }

  getClientTime(times: string) {
    let formatTime = times.replace(/-/g, '/');
    let Time = new Date(formatTime);
    let timestamp: number|string = Time.getTime();

    let nowTime = new Date().getTime();
    let thirdTime = nowTime - 24 * 60 * 60 * 1000 * 2;
    let day = new Date(thirdTime).getDay();
    day = day === 0 ? 7 : day;
    let endTimeStr = thirdTime - 24 * 60 * 60 * 1000;
    let firstTime = thirdTime - 24 * 60 * 60 * 1000 * (day - 1);

    let nowTimeDate = new Date(firstTime);
    nowTimeDate.setHours(0, 0, 0, 0);
    let dayStart = nowTimeDate.getTime();
    let endTime = new Date(endTimeStr);
    endTime.setHours(23, 59, 59, 999);
    let dayEnd = endTime.getTime();
    let dayMap: any = {
      0: "星期日",
      1: "星期一",
      2: "星期二",
      3: "星期三",
      4: "星期四",
      5: "星期五",
      6: "星期六",
    };

    let dataVal = new FormDate(new Date(timestamp));
    let getHour: string|number = parseInt(dataVal.getHour());
    if (getHour < 13) {
      getHour = "上午 " + getHour;
    }
    if (getHour >= 13) {
      getHour = "下午 " + (Number(getHour) - 12);
    }
    if (dayStart <= timestamp && timestamp <= dayEnd) {
      let day: number = new Date(timestamp).getDay();
      timestamp = `${dayMap[day]} ${getHour}:${dataVal.getMinute()}`;
    }
    // 前天
    else if (dayEnd < timestamp && timestamp <= dayEnd + 24 * 60 * 60 * 1000 - 1) {
      let day = new Date(timestamp).getDay();
      timestamp = `${dayMap[day]} ${getHour}:${dataVal.getMinute()}`;
    } else if (dayEnd + 24 * 60 * 60 * 1000 < timestamp && timestamp <= dayEnd + 24 * 60 * 60 * 1000 * 2 - 1) {
      timestamp = `昨天 ${getHour}:${dataVal.getMinute()}`;
    }
    // 今天
    else if (dayEnd + 24 * 60 * 60 * 1000 * 2 < timestamp && dayEnd + 24 * 60 * 60 * 1000 * 3 - 1) {
      timestamp = `${getHour}:${dataVal.getMinute()}`;
    } else {
      timestamp = `${dataVal.getYear()}-${dataVal.getMonth()}-${dataVal.getDay()} ${dataVal.getHour()}:${dataVal.getMinute()}`;
    }
    return timestamp;
  }

  getTimeInfo(): any {
    return this.getYear() + '-' + this.getMonth() + '-' + this.getDay()
      + ' ' + this.getHour() + ':' + this.getMinute() + ':' + this.getSecond();
  }

  // 计算时间显示 几天前 几个月前 几年前  24小时内显示 几分钟前和 几小时前
  getTimeBefore(time: string | number | Date) {
    if (!time) return "";

    class TimeBefore extends Date {
      time: string | number | Date;
      timeStr: number;

      constructor(time: string | number | Date) {
        super(time);
        this.time = time;
        if (!(time instanceof Date)) {
          this.time = new Date(time);
        }
        this.timeStr = (<Date>this.time).getTime();
      }

      getCurrentTimeStr() {
        return new Date().getTime();
      }

      getText() {
        if (this.timeStr > this.getCurrentTimeStr()) return "";

        const rules = [
          {timeStr: 60 * 60 * 1000, base: 60 * 1000, text: "分钟前"},
          {timeStr: 24 * 60 * 60 * 1000, base: 60 * 60 * 1000, text: "小时前"},
          {timeStr: 30 * 24 * 60 * 60 * 1000, base: 24 * 60 * 60 * 1000, text: "天前"},
          {timeStr: 12 * 30 * 24 * 60 * 60 * 1000, base: 30 * 24 * 60 * 60 * 1000, text: "个月前"},
          {timeStr: 12 * 12 * 30 * 24 * 60 * 60 * 1000, base: 12 * 30 * 24 * 60 * 60 * 1000, text: "个年前"},
        ];
        for (let info of rules) {
          if (this.getCurrentTimeStr() - this.timeStr < info.timeStr) {
            return Math.floor((this.getCurrentTimeStr() - this.timeStr) / info.base) + info.text;
          }
        }

        return Math.floor((this.getCurrentTimeStr() - this.timeStr) / rules[rules.length - 1].base)
          + rules[rules.length - 1].text;
      }
    }

    return new TimeBefore(time).getText();
  }

  // 获取当前时间前多少天的时间戳端 例如：2020-10-19 0:0:0:0 到 2020-10-22 23:59:59:59
  getTimeLimit(day: number): [number, number] {
    let endTime = new Date(this.now.setHours(23, 59, 59, 59)).getTime();
    let startTime = new Date(new Date((endTime - day*24*60*60*1000)).setHours(0,0,0,0)).getTime();
    return [startTime, endTime];
  }
  
  // 处理发布日期规则
  // 今天： 今日发布
  // 昨天： 昨日发布
  // 前天~30天：2天前发布
  // 30天前发布：超过30天~半年内
  // 1个月前发布
  // 2个月前
  // 超过半年~1年内：半年前发布
  // 超过1年：1年前
  // x年前
  getPushTime(unit = "发布"): string {
    const time = this.getTimeStr();
    // 今日时间段
    const todayList = new FormDate(new Date()).getTimeLimit(0);
    // 一天前时间段
    const oneDayBeforeList = new FormDate(new Date()).getTimeLimit(1);
    // 30天时间段
    const thirtyDayBeforeList = new FormDate(new Date()).getTimeLimit(29);
    // 半年内
    const halfYearList = new FormDate(new Date()).getTimeLimit(30 * 6 - 1);
    // 一年内
    const oneYearList = new FormDate(new Date()).getTimeLimit(30 * 12 - 1);
    // 一天的时间戳
    const dayStr = 24*60*60*1000;
    let text = "";
    do {
      if (todayList[0] <= time && time <= todayList[1]) {
        text = "今日";
        break;
      }
  
      if (oneDayBeforeList[0] <= time && time <= oneDayBeforeList[1]) {
        text = "昨日";
        break;
      }
  
      if (thirtyDayBeforeList[0] <= time && time <= thirtyDayBeforeList[1]) {
        text = parseInt(((todayList[0] - new Date(new Date(time).setHours(0,0,0,0)).getTime())/dayStr).toString())
          + "天前";
        break;
      }
  
      if (halfYearList[0] <= time && time <= halfYearList[1]) {
        text = parseInt(((todayList[0] - new Date(new Date(time).setHours(0,0,0,0)).getTime())/(dayStr*30)).toString())
          + "月前";
        break;
      }
  
      if (oneYearList[0] <= time && time <= oneYearList[1]) {
        text = "半年前";
        break;
      }
  
      text = parseInt(((todayList[0] - new Date(new Date(time).setHours(0,0,0,0)).getTime())/(dayStr*30*12)).toString())
        + "年前";
    }while (false);
    return text + unit;
  }

  // xx天后截止推广
  finalTime(): number {
    const endTime = this.getTimeStr();
    const nowTime = new Date().getTime();
    if (endTime <= nowTime) {
      return 0;
    }
    return Math.ceil((endTime-nowTime)/(24*60*60*1000));
  }
}
