import {Notification} from "@/common/Notification";
import { NotifyName } from '../Notify';
import {Model} from "@/common/model/Model";

class ModelMap {
  static map = new Map();

  static setMap(key: any, val: any) {
    this.map.set(key, val);
  }

  static getMap(key: any) {
    if (!this.map.has(key)) return null;

    return this.map.get(key);
  }
}

export abstract class DefaultModel<T, M> extends Model{
  key: string;
  constructor() {
    super();
    if (new.target.name === 'DefaultModel') {
      throw new Error('本类禁止实例化！');
    }

    this.key = "@key:" + new.target.name;
  }
  
  // 类的名字
  abstract getClassName(): string;
  
  // "@key:" + 类的名字
  private getKey(): string {
    if (!this.getClassName()) {
      alert("请配置类名");
      return "";
    }
    
    return "@key:" + this.getClassName();
  }

  // 是否保存到缓存
  protected getIsLocation(): boolean {
    return false;
  };

  protected getNcName(): NotifyName|null {
    return null;
  }

  sendNc(id?: T): void {
    if (this.getNcName() === null) return;

    new Notification().postNotification(this.getNcName() as NotifyName, this.getModel(id));
  }

  setModel(val: M, id?: T, needNc: boolean = true): void {
    if (this.getKey().indexOf("@key:") === -1) {
      alert("model的key格式不对");
      return;
    }
    
    let Obj = ModelMap.getMap(this.getKey()) || {};
    if (this.getIsLocation()) {
      Obj = JSON.parse(sessionStorage.getItem(this.getKey()) || "{}");
    }
    if (typeof id != 'undefined') {
      Obj[id] = val;
    } else {
      Obj = val;
    }
    if(this.getIsLocation()) {
      sessionStorage.setItem(this.getKey(), JSON.stringify(Obj));
    }
    ModelMap.setMap(this.getKey(), Obj);
    if (this.getNcName() !== null && needNc) {
      this.sendNc(id);
    }
  }

  getModel(id?: T): M|null {
    if (this.getKey().indexOf("@key:") === -1) {
      alert("model的key格式不对");
      return null;
    }
    
    let obj: any = {};
    if (!this.getIsLocation()) {
      obj = ModelMap.getMap(this.getKey()) || {};
    }else {
      obj = JSON.parse(sessionStorage.getItem(this.getKey()) || "{}");
    }
    let val: M|null = null;
    if(!id && (id as any) != '0') {
      val = obj || null;
      if(typeof obj == 'object' && Object.keys(obj).length === 0) {
        val = null;
      }
    }else {
      val = obj[id] || null;
    }
    return val;
  }

  removeModel() {
    let Obj = ModelMap.getMap(this.getKey()) || {};
    if (this.getKey()) {
      delete Obj[this.getKey()];
      ModelMap.setMap(this.getKey(), Obj);
    }
    if(this.getIsLocation()) {
      sessionStorage.removeItem(this.getKey());
    }
  }

  clearModel() {
    sessionStorage.clear();
  }

  addObserver(callback: (info: M) => void = () => {}) {
    if (this.getNcName() !== null) {
      new Notification<M>().addObserver(this.getNcName()!, (info: M)=> {
        callback(info);
      });
    }
  }

  removeObserver(): void {
    if (this.getNcName() === null) return;

    new Notification<M>().removeObserver(this.getNcName()!);
  }
}
