import {DefaultModel} from "@/common/model/DefaultModel";
import {ClassArray} from "@/common/utils/ClassArray";
import {APIResponse} from "@/common/net/API";
import {Notification} from "@/common/Notification";
import {NotifyName} from "@/common/Notify";


export abstract class ListModel<T, M extends ClassArray<APIResponse>> extends DefaultModel<T, M>{
  setModel(val: M, id?: T, needNc: boolean = true) {
    super.setModel(val, id, needNc);
    new Notification<M|null>().postNotification(NotifyName.listModel, this.getModel(id));
  }

  public addList(list: M, id?: T): void{
    if (!list || list.length === 0) {
      this.sendNc();
      new Notification<M|null>().postNotification(NotifyName.listModel, this.getModel(id));
      return;
    }

    let model = this.getModel(id) || new ClassArray<APIResponse>(APIResponse);
    for (let info of list) {
      model.push(info);
    }
    this.setModel(model as M, id);
  }

  public clearList(): void {
    this.setModel(new ClassArray<APIResponse>(APIResponse) as M);
  }

  public removeList(list: M, listKey: string): void {
    if (list.length === 0) return;

    let map: Map<any, any> = new Map<any, any>();
    for (let info of list) {
      map.set((<any>info)[listKey], info);
    }
    let model = this.getModel() || new ClassArray<APIResponse>(APIResponse);
    let newModel: ClassArray<APIResponse> = new ClassArray<APIResponse>(APIResponse);
    for (let info of model) {
      if (map.has((<any>info)[listKey])) continue;

      newModel.push(info);
    }
    this.setModel(newModel as M);
  }

  public updateList(list: M, listKey: string, id?: T): void{
    if (list.length === 0) return;

    let map: Map<any, any> = new Map<any, any>();
    for (let info of list) {
      map.set((<any>info)[listKey], info);
    }
    let model = this.getModel(id) || new ClassArray<APIResponse>(APIResponse);
    let newModel: ClassArray<APIResponse> = new ClassArray<APIResponse>(APIResponse);
    for (let info of model) {
      newModel.push(map.get((<any>info)[listKey]) || info);
    }
    this.setModel(newModel as M, id);
  }
  
  abstract getClassName(): string;
}
