import {Config} from "@/config/Config";

class RequestTimeOut<T> {
  timeoutFun: null | any = null;
  timeoutTime: number = 20000;

  public send<T>(): Promise<T> {
    return new Promise((resolve, reject) => {
      this.timeoutFun = setTimeout((info: any) => {
        this.timeoutFun = null;
        reject("超时");
      }, this.timeoutTime)
    })
  }
}

export class APIResponse {}

export class APIRequest {}

export class APIErrorResponse extends APIResponse {
  msg: string | null = null;

  constructor(msg: string | null) {
    super();
    if (msg) {
      this.msg = msg;
    }
  }
}

interface RequestInterface {
  [key: string]: any;
}

export abstract class API<T extends APIResponse> {
  request: RequestInterface = {};
  // 成功的返回值
  private response: T | undefined;
  // 失败的返回值
  private failResponse: any;

  constructor(request: APIRequest) {
    // if (!(request instanceof APIRequest)) {
    //   throw new Error("请求参数必须是APIRequest的派生类")
    // }

    let dealRequest: any = request;
    for (let k in dealRequest) {
      if (dealRequest[k] === undefined || dealRequest[k] === null) continue;

      this.request[k] = dealRequest[k];
    }
  }

  abstract getApiDomain(): Config;

  abstract getUrl(): string;

  // 是否需要clientKey
  getNeedClientKey(): boolean {
    return false;
  }

  getHeader(): any {
    return {
      "U-FS": "uuWeidian",
      "webPass": "true",
    };
  };

  // 是否需要特殊处理域名
  needDealDomain(): boolean {
    return true;
  }

  // 是否需要表单格式提交
  getNeedFormData(): boolean {
    return false;
  }

  abstract run(): Promise<T>;

  send(): Promise<T> {
    return new Promise((resolve, reject) => {
      if (!this.getUrl()) {
        reject(new APIErrorResponse("暂无url"));
        return;
      }

      // Promise.race([this.run(), new RequestTimeOut<string>().send()])
      return this.run()
        .then((response: T) => {
          return resolve(response);
        })
        .catch((err: T) => {
          reject(err);
        });
    })
  }

  // 获取成功的返回值
  getResponse(): T | undefined {
    return this.response;
  }
  // 更新成功返回值
  setResponse(response: T | undefined): void {
    this.response = response;
  }

  // 获取失败返回值
  getFailResponse(): any {
    return this.failResponse || {};
  }
  // 更新失败返回值
  setFailResponse(response: any): void {
    this.failResponse = response;
  }

  // 是否需要底层弹出toast
  isShowFailToast(): boolean {
    return true;
  }
  
  // 平台类型是否追加到header中
  addPlateToHeader(): boolean {
    return true;
  }
  
  // 是否需要使用php redirect 方式请求接口
  useRedirect(): boolean {
    return false;
  }
  
  // 使用传入的请求地址去请求
  getAction(): string {
    return "";
  }
  
  protected abstract parse(json: object): T;

  protected abstract setModel(): Promise<void>;
}
