import {Const} from "@/common/net/Const";
// @ts-ignore
import axios from "axios";
import {Config} from "@/config/Config";

export enum RequestMethodEnum {
 POST = "POST",
 GET = "GET",
}
export class WxRequest<T> {
  private _successStatus: number = 1;
  private url: string = "";
  private domain: Config = Config.uuwebPhp;
  private headers: object = {};
  private requestOptions: any = {};
  private method: RequestMethodEnum = RequestMethodEnum.POST;
  private allResponse: any = {};
  // 是否需要传表单
  private needFormData: boolean = false;
  // 是否需要再header中增加plateformType
  private headerNeedAddPlate: boolean = false;
  // 是否需要使用php redirect 方式请求接口
  private useRedirect: boolean = false;
  // 直接使用action地址去请求地址 写插件跨域用的
  private action: string = "";
  
  constructor(domain: Config, url: string = "", headers: object = {},
              method: RequestMethodEnum, requestOptions: object = {},
              needFormData = false, headerNeedAddPlate: boolean = true,
              needDealDomain: boolean = true, useRedirect: boolean = false, action: string) {
    this.url = url;
    this.domain = domain;
    this.headers = headers;
    this.method = method;
    this.requestOptions = requestOptions;
    this.needFormData = needFormData;
    this.headerNeedAddPlate = headerNeedAddPlate;
    this.useRedirect = useRedirect;
    this.action = action;
  }

  private createFormDataRequest(): FormData {
    let form = new FormData();
    for (let k in  (this.requestOptions || {})) {
      form.append(k, ((this.requestOptions || {}) as any)[k]);
    }
    return form;
  }

  public request(): Promise<object> {
    const instance = axios.create({
      withCredentials: (<any>this.headers).withCredentials === undefined
        ? false
        : (<any>this.headers).withCredentials
    });
    let instanceConfig: any = {
      method: this.method,
      url: this.url,
      headers: this.headers
    };
    // dontNeedToRequestHead==1
    // plateformType不需要在header里面
    if (!this.headerNeedAddPlate) {
      this.requestOptions["dontNeedToRequestHead"] = 1;
    }
    if (this.method === RequestMethodEnum.GET) {
      if (this.useRedirect) {
        this.requestOptions["redirectApi"] = this.domain + this.url;
        this.requestOptions["redirectApi"] = this._replaceUrl(this.requestOptions["redirectApi"]);
        instanceConfig.url = this.action || (Config.uuwebPhp + "/api/Api/requestGetRedirect");
      }else {
        instanceConfig.url = this.domain + this.url;
      }
      instanceConfig.params = this.needFormData ? this.createFormDataRequest() : this.requestOptions;
      instanceConfig.url = this._replaceUrl(instanceConfig.url);
    }
    if (this.method === RequestMethodEnum.POST) {
      if (this.useRedirect) {
        this.requestOptions["javaApi"] = this.domain + this.url;
        instanceConfig.url = this.action || (Config.uuwebPhp + "/api/Api/requestRedirect");
        if (this.needFormData) {
          instanceConfig.url = this.action || (Config.uuwebPhp + "/api/Api/requestPostRedirect");
        }
        this.requestOptions["javaApi"] = this._replaceUrl(this.requestOptions["javaApi"]);
      }else {
        instanceConfig.url = this.domain + this.url;
      }
      instanceConfig.data = this.needFormData ? this.createFormDataRequest() : this.requestOptions;
      instanceConfig.url = this._replaceUrl(instanceConfig.url);
    }
    return instance(instanceConfig).then((res: any) => {
      let model: any = res;
      model = model.data || {};
      if (typeof model.errCode !== "undefined" && model.errCode !== Const.success) {
        return Promise.reject(model);
      }

      if (typeof model.errCode === "undefined" && typeof model.status !== "undefined" && model.status !== Const.success && model.status !== 1) {
        return Promise.reject(model);
      }

      if (typeof model.STATUS !== "undefined" && model.STATUS !== this._successStatus) {
        return Promise.reject(model);
      }

      if (model.statusCode && model.statusCode !== Const.success) {
        return Promise.reject(model);
      }

      if (model.code && model.code !== Const.success) {
        return Promise.reject(model);
      }
  
      this.allResponse = model;
      let val: object;
      val = model.data || model.DATA || {};
  
      let info = val instanceof Array ? {list: val} : val;
      if (typeof val === "string") {
        info = {data: val};
      }
      if (typeof val === "object" && Object.keys(val).length === 0 && typeof model === "object") {
        info = model;
      }
      return info;
    }).catch((err: any) => {
      return Promise.reject(err);
    });
  }
  
  // 替换//为/
  private _replaceUrl(url: string): string {
    let urls = url.split("://");
    if (urls.length === 2) {
      url = urls[0] + "://" + urls[1].replace(new RegExp("//", 'g'), "/");
    }
    return url;
  }
}
