import {APIRequest, APIResponse} from "@/common/net/API";
import {PostRequest} from "@/common/net/PostRequest";
import {Config} from "@/config/Config";
import {Json, SerializedName} from "@/common/utils/Json";
import {Const} from "@/common/net/Const";

export class ImageUploadRequest extends APIRequest{
  file: any = "";
}

export class ImageUploadResponse extends APIResponse{
  // 全地址
  fullUrl: string = "";
  // 文件名称
  @SerializedName("saveName")
  name: string = "";
  // 源文件名字
  oriName: string = "";
  // 前缀
  domain: string = "";
  // 文件大小
  size: number = 0;
}

export class ImageUpload extends PostRequest<ImageUploadResponse>{
  // 必传请求服务器地址
  private _action: string = "";
  constructor(props:ImageUploadRequest, action: string) {
    super(props);
    this._action = action;
  }

  getHeader(): any {
    let header = super.getHeader();
    header["Content-Type"] = "multipart/form-data";
    return header;
  }

  getApiDomain(): Config {
    return "";
  }

  getUrl(): string {
    return this._action;
  }
  
  getNeedFormData(): boolean {
    return true;
  }
  
  protected parse(json: any): ImageUploadResponse {
    let fileInfo = !json.data ? {} : json.data[0] || {};
    if (json.code !== Const.success) {
      fileInfo = {};
    }
    return new Json().fromJson(fileInfo,ImageUploadResponse);
  }

  protected setModel(): Promise<void> {
    return Promise.resolve(undefined);
  }
}
