const path = require('path');
const defaultSettings = require('./src/settings.js');

function resolve(dir) {
  return path.join(__dirname, dir)
}

const name = defaultSettings.title || 'vue Element Admin'; // page title
const port = process.env.port || process.env.npm_config_port || 9529; // dev port
module.exports = {
  publicPath: "./",
  outputDir: 'ossUpload',
  assetsDir: 'static',
  // 指定生成的 index.html 的输出路径 (相对于 outputDir)。也可以是一个绝对路径。
  indexPath: 'index.html',
  lintOnSave: process.env.NODE_ENV !== 'production',
  runtimeCompiler: true,
  productionSourceMap: false,
  configureWebpack: {
    // provide the app's title in webpack's name field, so that
    // it can be accessed in index.html to inject the correct title.
    name: name,
    resolve: {
      alias: {
        '@': resolve('src'),
        'vue$': 'vue/dist/vue.esm.js'    // 处理vue节点
      }
    }
  },
  devServer: {
    port: port,
    open: true,
    overlay: {
      warnings: false,
      errors: true
    },
    proxy: {
      '/uuwebPhp': {
        // target: "http://uuweb.hftsoft.com",
        target: "http://uuweb.uj.cn",
        changeOrigin: true,
        pathRewrite: {
          '^/uuwebPhp': '/'
        },
        secure: false
      },
      '/hftJava': {
        target: "https://hft.uj.cn",
        changeOrigin: true,
        pathRewrite: {
          '^/hftJava': '/'
        },
        secure: false
      }
    },
  },
  // 一定要加这个  要不然css浏览器前缀没办法自动加
  css: {
    loaderOptions: {
      postcss: {
        plugins: [
          require('autoprefixer')({ // 配置使用 autoprefixer
            // browsers: ['last 20 versions'],
            overrideBrowserslist: ['last 20 versions'] // 记得这里要把 browsers 改为 overrideBrowserslist，autoprefixer 新版本的写法有变
          }),
          // require('postcss-plugin-px2rem')({ // 配置使用 postcss-plugin-px2rem 把 px 单位转成 rem，当然项目 main.js 中要相应 import 'amfe-flexible' 作为移动端的自适应解决方案
          //   rootValue: 75,
          //   unitPrecision: 8,
          //   propWhiteList: [],
          //   propBlackList: [],
          //   selectorBlackList: [],
          //   ignoreIdentifier: false,
          //   replace: true,
          //   mediaQuery: false,
          //   minPixelValue: 2
          // })
        ]
      }
    }
  },
  chainWebpack: config => {
    config
      .entry('main')
      .add('babel-polyfill')
  },
};
